-- If LuaRocks is installed, make sure that packages installed through it are
-- found (e.g. lgi). If LuaRocks is not installed, do nothing.
pcall(require, "luarocks.loader")

-- Standard awesome library
local gears = require("gears")
local gmath = require("gears.math")
local awful = require("awful") 
require("awful.autofocus")

-- Widget and layout library
local wibox = require("wibox")

-- Theme handling library
local beautiful = require("beautiful")

-- Notification library
local naughty = require("naughty")
local menubar = require("menubar")
local hotkeys_popup = require("awful.hotkeys_popup")

-- Enable hotkeys help widget for VIM and other apps
-- when client with a matching name is opened:
require("awful.hotkeys_popup.keys")


--------------------------
-- -- ERROR HANDLING -- --
--------------------------
-- Check if awesome encountered an error during startup and fell back to
-- another config (This code will only ever execute for the fallback config)
if awesome.startup_errors then
    naughty.notify({ preset = naughty.config.presets.critical,
                     title = "Oops, there were errors during startup!",
                     text = awesome.startup_errors })
end

-- Handle runtime errors after startup
do
    local in_error = false
    awesome.connect_signal("debug::error", function (err)
        -- Make sure we don't go into an endless error loop
        if in_error then return end
        in_error = true

        naughty.notify({ preset = naughty.config.presets.critical,
                         title = "Oops, an error happened!",
                         text = tostring(err) })
        in_error = false
    end)
end

--------------------
------------
-- -- VARIABLE DEFINITIONS -- --
--------------------------------
-- Themes define colours, icons, font and wallpapers.
-- beautiful.init(gears.filesystem.get_themes_dir() .. "default/theme.lua")
beautiful.init(gears.filesystem.get_configuration_dir() .. 'themes/custom.lua')

-- default terminal
terminal = "kitty"

-- default editor
editor = os.getenv("EDITOR") or "code"

-- Default modkey.
modkey = "Mod4"

-- Table of layouts to cover with awful.layout.inc, order matters.
awful.layout.layouts = {
    awful.layout.suit.tile,
    awful.layout.suit.spiral.dwindle,
    awful.layout.suit.corner.nw,
}


----------------
-- -- MENU -- --
----------------
-- Benutzer
useractions = {
    { "Abmelden", function() awesome.quit() end },
    { "Neustarten", function() awful.spawn("systemctl reboot") end },
    { "Energie sparen", function() awful.spawn("systemctl suspend") end },
    { "Herunterfahren", function() awful.spawn("systemctl poweroff") end },
}

-- Window Manager
myawesomemenu = {
   { "hotkeys", function() hotkeys_popup.show_help(nil, awful.screen.focused()) end },
   { "manual", terminal .. " -e man awesome" },
   { "restart", awesome.restart },
   { "quit", function() awesome.quit() end },
}

-- Hauptmenü
mymainmenu = awful.menu({ 
    items = {
        { "Benutzer", useractions },
        { "Window Manager", myawesomemenu },
        { "Terminal", terminal }
    }
})


-----------------
-- -- WIBAR -- --
-----------------
-- Create a wibox for each screen and add it
local taglist_buttons = gears.table.join(
    awful.button({ }, 1, function(t) t:view_only() end),
    awful.button(
        { modkey }, 1,
        function(t)
            if client.focus then
                client.focus:move_to_tag(t)
            end
        end
    ),
    awful.button({ }, 3, awful.tag.viewtoggle),
    awful.button(
        { modkey }, 3,
        function(t)
            if client.focus then
                client.focus:toggle_tag(t)
            end
        end
    ),
    awful.button({ }, 4, function(t) awful.tag.viewnext(t.screen) end),
    awful.button({ }, 5, function(t) awful.tag.viewprev(t.screen) end)
)

local tasklist_buttons = gears.table.join(
    awful.button(
        { }, 1,
        function (c)
            if c == client.focus then
                c.minimized = true
            else
                c:emit_signal(
                    "request::activate",
                    "tasklist",
                    {raise = true}
                )
            end
        end
    ),
    awful.button(
        { }, 3, 
        function (c) awful.menu.client_list({ theme = { width = 250 } }) end
    ),
    awful.button( { }, 4, function () awful.client.focus.byidx(1) end ),
    awful.button( { }, 5, function () awful.client.focus.byidx(-1) end)
)


----------------------------------
-- -- HINTERGRUNDBILD SETZEN -- --
----------------------------------
local function set_wallpaper(s)
    -- Wallpaper
    if beautiful.wallpaper then
        local wallpaper = beautiful.wallpaper
        -- If wallpaper is a function, call it with the screen
        if type(wallpaper) == "function" then
            wallpaper = wallpaper(s)
        end
        gears.wallpaper.maximized(wallpaper, s, true)
    end
end

-- Re-set wallpaper when a screen's geometry changes (e.g. different resolution)
screen.connect_signal("property::geometry", set_wallpaper)


-------------------------------
-- -- MONITORE EINRICHTEN -- --
-------------------------------
awful.screen.connect_for_each_screen(function(s)
    -- Wallpaper einrichten
    set_wallpaper(s)

    -- Arbeitsflächen einstellen
    awful.tag({ " Ⅰ ", " Ⅱ ", " Ⅲ ", " Ⅳ ", " Ⅴ " }, s, awful.layout.layouts[1])

    -- Create a taglist widget
    s.mytaglist = awful.widget.taglist {
        screen  = s,
        filter  = awful.widget.taglist.filter.all,
        buttons = taglist_buttons
    }

    -- Create a tasklist widget
    s.mytasklist = awful.widget.tasklist {
        screen  = s,
        filter  = awful.widget.tasklist.filter.currenttags,
        buttons = tasklist_buttons
    }

    -- Create the wibox
    s.mywibox = awful.wibar({ position = "top", screen = s })

    s.my_clock = wibox.widget {
        format = '  %d.%m.%Y  ◆  %H:%M:%S  ',
        refresh = 1,
        widget = wibox.widget.textclock
    }
    
    s.my_clock:buttons(gears.table.join(
        s.my_clock:buttons(),
        awful.button({}, 1, nil, function()
            awful.spawn("kitty")
        end)
    ))

    -- Add widgets to the wibox
    s.mywibox:setup {
        layout = wibox.layout.align.horizontal,
        {
            -- Left widgets
            layout = wibox.layout.fixed.horizontal,
            s.mytaglist
        },
        -- Middle widget
        s.mytasklist,
        {
            -- Right widgets
            layout = wibox.layout.fixed.horizontal,
            wibox.widget.systray(),
            spacer,
            s.my_clock,
        },
    }
end)


--------------------------
-- -- MOUSE BINDINGS -- --
--------------------------
root.buttons(gears.table.join(
    awful.button({ }, 3, function () mymainmenu:toggle() end),
    awful.button({ }, 4, awful.tag.viewnext),
    awful.button({ }, 5, awful.tag.viewprev)
))


-----------------------------
-- -- KEYBOARD BINDINGS -- --
-----------------------------
globalkeys = gears.table.join(
    -- reload awesome
    awful.key(
        { modkey, "Control" }, "r",
        awesome.restart,
        {description = "reload awesome", group = "awesome"}
    ),

    -- run prompt
    awful.key(
        { modkey }, "r",
        function () awful.spawn("rofi -show drun") end,
        {description = "run prompt", group = "Launcher"}
    ),

    ---------------------
    -- WINDOW CONTROLL --
    ---------------------
    -- move view to left tag
    awful.key(
        { modkey }, "Left",
        awful.tag.viewprev,
        {description = "Arbeitsfläche links", group = "Workspace"}
    ),

    -- move view to right tag
    awful.key(
        { modkey }, "Right",
        awful.tag.viewnext,
        {description = "Arbeitsfläche rechts", group = "Workspace"}
    ),

    -- move focus down
    awful.key(
        { modkey }, "Down",
        function () awful.client.focus.byidx(1) end,
        {description = "Fokus vorheriges Fenster", group = "Fenster"}
    ),

    -- move focus up
    awful.key(
        { modkey }, "Up",
        function () awful.client.focus.byidx(-1) end,
        {description = "Fokus nächstes Fenster", group = "Fenster"}
    ),
    
    -- move window to left tag
    awful.key(
        { modkey, "Ctrl" }, "Left",
        function(c) 
            local c = client.focus
            if not c then return end
            local t = c.screen.selected_tag
            local tags = c.screen.tags
            local idx = t.index
            local newtag = tags[gmath.cycle(#tags, idx-1)]
            c:move_to_tag(newtag)
            awful.tag.viewprev()
        end,
        {description = "Fenster nach links", group = "Workspace"}
    ),

    -- move window to right tag
    awful.key(
        { modkey, "Ctrl" }, "Right",
        function(c) 
            local c = client.focus
            if not c then return end
            local t = c.screen.selected_tag
            local tags = c.screen.tags
            local idx = t.index
            local newtag = tags[gmath.cycle(#tags, idx+1)]
            c:move_to_tag(newtag)
            awful.tag.viewnext()
        end,
        {description = "Fenster nach rechts", group = "Workspace"}
    ),

    -- move window down
    awful.key(
        { modkey, "Ctrl" }, "Down",
        function () awful.client.swap.byidx(1) end,
        {description = "Fenster nach unten schieben", group = "Fenster"}
    ),
    
    -- move window up
    awful.key(
        { modkey, "Ctrl" }, "Up",
        function () awful.client.swap.byidx( -1) end,
        {description = "Fenster nach oben schieben", group = "Fenster"}
    ),
    
    -- move window to next display
    awful.key(
        { modkey, "Ctrl" }, "o", 
        function () awful.screen.focus_relative( 1) end,
        {description = "Fokus auf nächsten Monitor", group = "Monitor"}
    ),

    ---------------------
    -- PROGRAM STARTER --
    ---------------------
    -- Terminal
    awful.key({ modkey,           }, "Return", function () awful.spawn(terminal) end,
              {description = "Terminal", group = "Launcher"}),

    -- Explorer
    awful.key({ modkey }, "e",     function () awful.spawn("nemo") end,
              {description = "Explorer", group = "Launcher"}),

    -- Browser
    awful.key({ modkey }, "w",     function () awful.spawn("firefox") end,
              {description = "Web browser", group = "Launcher"}),

    -- Email
    awful.key({ modkey }, "t",     function () awful.spawn("thunderbird") end,
              {description = "Email client", group = "Launcher"})
)

---------------------
-- CLIENT CONTROLL --
---------------------
clientkeys = gears.table.join(
    -- toggle floating
    awful.key(
        { modkey, "Control" }, "space",
        awful.client.floating.toggle,
        {description = "toggle floating", group = "client"}
    ),

    -- toggle fullscreen
    awful.key(
        { modkey }, "m",
        function (c)
            c.fullscreen = not c.fullscreen
            c:raise()
        end,
        {description = "toggle fullscreen", group = "Fenster"}
    ),

    -- close window
    awful.key(
        { modkey }, "q",
        function (c) c:kill() end,
        { description = "close", group = "Fenster" }
    ),

    -- move window to next screen
    awful.key(
        { modkey }, "o",
        function (c) c:move_to_screen() end,
        { description = "move to screen", group = "client" }
    ),

    -- toggle maximize
    awful.key(
        { modkey }, "space",
        function (c)
            c.maximized = not c.maximized
            c:raise()
        end,
        { description = "(un)maximize", group = "client"}
    )
)

-- Set keys
root.keys(globalkeys)


clientbuttons = gears.table.join(
	awful.button({ }, 1, function (c)
		c:emit_signal("request::activate", "mouse_click", {raise = true})
	end),
	awful.button({ modkey }, 1, function (c)
		c:emit_signal("request::activate", "mouse_click", {raise = true})
		awful.mouse.client.move(c)
	end),
	awful.button({ modkey }, 3, function (c)
		c:emit_signal("request::activate", "mouse_click", {raise = true})
		awful.mouse.client.resize(c)
	end)
)

-----------------
-- -- RULES -- --
-----------------
-- Rules to apply to new clients (through the "manage" signal).
awful.rules.rules = {
    { -- All clients will match this rule.
        rule = { },
        properties = { 
            border_width = beautiful.border_width,
            border_color = beautiful.border_normal,
            focus = awful.client.focus.filter,
            raise = true,
            keys = clientkeys,
            buttons = clientbuttons,
            screen = awful.screen.preferred,
            placement = awful.placement.no_overlap+awful.placement.no_offscreen
        }
    },
    { -- Floating clients.
        rule_any = {
            instance = {
                "DTA",  -- Firefox addon DownThemAll.
                "copyq",  -- Includes session name in class.
                "pinentry",
            },
            class = {
                "Arandr",
                "Blueman-manager",
                "Gpick",
                "Kruler",
                "MessageWin",  -- kalarm.
                "Sxiv",
                "Tor Browser", -- Needs a fixed window size to avoid fingerprinting by screen size.
                "Wpa_gui",
                "veromix",
                "xtightvncviewer"
            },

            -- Note that the name property shown in xprop might be set slightly after creation of the client
            -- and the name shown there might not match defined rules here.
            name = {
                "Event Tester",  -- xev.
            },
            role = {
                "AlarmWindow",  -- Thunderbird's calendar.
                "ConfigManager",  -- Thunderbird's about:config.
                "pop-up",       -- e.g. Google Chrome's (detached) Developer Tools.
            }
        },
        properties = { floating = true }
    },
}


-------------------
-- -- SIGNALS -- --
-------------------
-- Signal function to execute when a new client appears.
client.connect_signal("manage", function (c)
    -- Set the windows at the slave,
    -- i.e. put it at the end of others instead of setting it master.
    -- if not awesome.startup then awful.client.setslave(c) end

    if awesome.startup
      and not c.size_hints.user_position
      and not c.size_hints.program_position then
        -- Prevent clients from being unreachable after screen count changes.
        awful.placement.no_offscreen(c)
    end
end)

-- Enable sloppy focus, so that focus follows mouse.
client.connect_signal("mouse::enter", function(c)
    c:emit_signal("request::activate", "mouse_enter", {raise = false})
end)

client.connect_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.connect_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)


---------------------------
-- -- DYNAMIC BORDERS -- --
---------------------------
beautiful.fullscreen_hide_border = true
beautiful.maximized_hide_border = true

local function set_border(c)
    local s = awful.screen.focused()
    if (#s.tiled_clients == 1 and not c.floating)
        or (s.selected_tag and s.selected_tag.name == "max")
    then
        c.border_width = 0
    else
        c.border_width = beautiful.border_width
    end
end

client.connect_signal("focus", set_border)
client.connect_signal("request::border", set_border)
client.connect_signal("property::maximized", set_border)